
let preventInstance = {};

function startJustRead(tab) {
  if (tab) {
    executeScripts(tab.id);
  } else {
    chrome.tabs.query({ currentWindow: true, active: true }, (tabArray) => {
      if (tabArray.length) executeScripts(tabArray[0].id);
    });
  }
}

function executeScripts(tabId) {
  if (preventInstance[tabId]) return;

  preventInstance[tabId] = true;
  setTimeout(() => delete preventInstance[tabId], 10000);

  // Add a badge to signify the extension is in use
  chrome.action.setBadgeBackgroundColor({ color: [242, 38, 19, 230] });
  chrome.action.setBadgeText({ text: "on" });

  // Load our external scripts, then our content script
  chrome.scripting
    .executeScript({
      target: { tabId: tabId, allFrames: false },
      files: [
        "/external-libraries/readability.js",
        "/external-libraries/purify.min.js",
      ],
    })
    .then(() => {
      chrome.scripting.executeScript({
        target: { tabId: tabId, allFrames: false },
        files: ["content_script.js"],
      });

      setTimeout(function () {
        chrome.action.setBadgeText({ text: "" });
      }, 1000);
    });
}

// Listen for the extension's click
chrome.action.onClicked.addListener(startJustRead);

// Listen for the keyboard shortcut
chrome.commands.onCommand.addListener(function (command) {
  if (command == "open-just-read") startJustRead();
});