// prettier-ignore
(function () {

let leavePres = true;
let removeOrigContent;
let pageSelectedContainer;
  removeOrigContent = true;
/*chrome.storage.sync.get(null, function (result) {
 // launch();
});*/

/////////////////////////////////////
// Generic helper functions
/////////////////////////////////////

// Add :scope functionality to QS & QSA
(function (doc, proto) {
  try {
    // Check if browser supports :scope natively
    doc.querySelector(":scope body");
  } catch (err) {
    // Polyfill native methods if it doesn't
    ["querySelector", "querySelectorAll"].forEach(function (method) {
      const nativ = proto[method];
      proto[method] = function (selectors) {
        if (/(^|,)\s*:scope/.test(selectors)) {
          // Only if selectors contains :scope
          const id = this.id; // Remember current element id
          this.id = "ID_" + Date.now(); // Assign new unique id
          selectors = selectors.replace(/((^|,)\s*):scope/g, "$1#" + this.id); // Replace :scope with #ID
          const result = doc[method](selectors);
          this.id = id; // Restore previous id
          return result;
        } else {
          return nativ.call(this, selectors); // Use native code for other selectors
        }
      };
    });
  }
})(window.document, Element.prototype);


// Mute a singular HTML5 element
function muteMe(elem) {
  elem.muted = true;
  elem.pause();
}

// Try to mute all video and audio elements on the page
function mutePage() {
  document.querySelectorAll("video").forEach((video) => muteMe(video));
  document.querySelectorAll("audio").forEach((audio) => muteMe(audio));
}

// Generate a random UUID (string)
// Example: 9ae68c40-0431-4031-afa0-3016ae50ad5d
function uuidv4() {
  return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, (c) =>
    (
      c ^
      (crypto.getRandomValues(new Uint8Array(1))[0] & (15 >> (c / 4)))
    ).toString(16)
  );
}

/////////////////////////////////////
// Extension-related helper functions
/////////////////////////////////////

// From https://stackoverflow.com/a/14824756/2065702
function isRTL(s) {
  const ltrChars =
      "A-Za-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02B8\u0300-\u0590\u0800-\u1FFF" +
      "\u2C00-\uFB1C\uFDFE-\uFE6F\uFEFD-\uFFFF",
    rtlChars = "\u0591-\u07FF\uFB1D-\uFDFD\uFE70-\uFEFC",
    rtlDirCheck = new RegExp("^[^" + ltrChars + "]*[" + rtlChars + "]");

  return rtlDirCheck.test(s);
}

function getArticleTitle() {
  // Get the page's title
  let title;

  if (titleSelector && document.querySelector(titleSelector)) {
    const elem = document.querySelector(titleSelector);
    title = elem.innerText;
    elem.dataset.simpleDelete = true; // Flag it for removal later
  } else if (document.head.querySelector("title")) {
    title = document.head.querySelector("title").innerText;

    // Get the part before the first — if it exists
    if (title.indexOf(" — ") > 0) {
      return title.substr(0, title.indexOf(" — "));
    }

    // Get the part before the first – if it exists
    if (title.indexOf(" – ") > 0) {
      return title.substr(0, title.indexOf(" – "));
    }

    // Get the part before the first - if it exists DIFFERENT THAN ABOVE CHARACTER
    if (title.indexOf(" - ") > 0) {
      return title.substr(0, title.indexOf(" - "));
    }

    // Get the part before the first | if it exists
    if (title.indexOf(" | ") > 0) {
      return title.substr(0, title.indexOf(" | "));
    }

    // Get the part before the first : if it exists
    if (title.indexOf(" : ") > 0) {
      return title.substr(0, title.indexOf(" : "));
    }
  } else {
    title = "Unknown title";
  }

  return title;
}


function getArticleContainer() {
  let selectedContainer;

  if (contentSelector && document.querySelector(contentSelector)) {
    selectedContainer = document.querySelector(contentSelector);
  } else if (document.head.querySelector("meta[name='articleBody'")) {
    selectedContainer = document.createElement("div");
    selectedContainer.innerHTML = DOMPurify.sanitize(
      document.head
        .querySelector("meta[name='articleBody'")
        .getAttribute("content")
    );
  } else {
    const numWordsOnPage = document.body.innerText.match(/\S+/g).length;
    let ps = document.body.querySelectorAll("p");

    // Find the paragraphs with the most words in it
    let pWithMostWords = document.body,
      highestWordCount = 0;

    if (ps.length === 0) {
      ps = document.body.querySelectorAll("div");
    }

    ps.forEach((p) => {
      if (
        checkAgainstBlacklist(p, 3) && // Make sure it's not in our blacklist
        p.offsetHeight !== 0
      ) {
        //  Make sure it's visible on the regular page
        const myInnerText = p.innerText.match(/\S+/g);
        if (myInnerText) {
          const wordCount = myInnerText.length;
          if (wordCount > highestWordCount) {
            highestWordCount = wordCount;
            pWithMostWords = p;
          }
        }
      }

      // Remove elements in JR that were hidden on the original page
      if (p.offsetHeight === 0) {
        p.dataset.simpleDelete = true;
      }
    });

    // Keep selecting more generally until over 2/5th of the words on the page have been selected
    selectedContainer = pWithMostWords;
    let wordCountSelected = highestWordCount;

    while (
      wordCountSelected / numWordsOnPage < 0.4 &&
      selectedContainer != document.body &&
      selectedContainer.parentElement.innerText
    ) {
      selectedContainer = selectedContainer.parentElement;
      wordCountSelected = selectedContainer.innerText.match(/\S+/g).length;
    }

    // Make sure a single p tag is not selected
    if (selectedContainer.tagName === "P") {
      selectedContainer = selectedContainer.parentElement;
    }
  }

  return selectedContainer;
}

// Remove what we added (besides styles)
function closeOverlay() {
  // Refresh the page if the content has been removed
  if (removeOrigContent) {
    const url = new URL(window.location);
    url.searchParams.delete("jr");
    window.location.replace(url);
  }

  // Remove the GUI if it is open
/*  if (datGUI) {
    datGUI.destroy();
    datGUI = undefined;
  }*/

  // Fade out
  simpleArticle.classList.add("simple-fade-up");


  // Reset our variables
  pageSelectedContainer = null;
  simpleArticleIframe = undefined;
  editBar = undefined;

  setTimeout(function () {
    // Enable scroll
    document.documentElement.classList.remove("simple-no-scroll");

    // Update our background script
    chrome.runtime.sendMessage({ lastClosed: Date.now() });

    // Remove our overlay
    simpleArticle.parentElement.removeChild(simpleArticle);
    simpleArticle = undefined;
  }, 100); // Make sure we can animate it
}

// Handle link clicks
function linkListener(e) {
  if (!simpleArticleIframe.body.classList.contains("simple-deleting")) {
    // Don't change the top most if it's not in the current window
    if (
      e.ctrlKey ||
      e.shiftKey ||
      e.metaKey ||
      (e.button && e.button == 1) ||
      this.target === "about:blank" ||
      this.target === "_blank"
    ) {
      return; // Do nothing
    }

    // Don't change the top most if it's referencing an anchor in the article
    const hrefArr = this.href.split("#");

    if (
      hrefArr.length < 2 || // No anchor
      (hrefArr[0] !== top.window.location.href.split("#")[0] && // Anchored to an ID on another page
        hrefArr[0] !== "about:blank" &&
        hrefArr[0] !== "_blank") ||
      (simpleArticleIframe.getElementById(hrefArr[1]) == null && // The element is not in the article section
        simpleArticleIframe.querySelector("a[name='" + hrefArr[1] + "']") ==
          null &&
        hrefArr[1] !== "_")
    ) {
      top.window.location.href = this.href; // Regular link
    } else {
      // Anchored to an element in the article
      e.preventDefault();
      e.stopPropagation();

      if (hrefArr[1].startsWith("jr-")) {
        simpleArticleIframe.getElementById(hrefArr[1]).scrollIntoView(true);
        let backArrow = simpleArticleIframe.querySelector(
          this.id + " .back-to-ref"
        );
        backArrow.dataset.scrollPos = simpleArticleIframe.scrollTop;
      } else {
        top.window.location.hash = hrefArr[1];
        simpleArticleIframe.defaultView.location.hash = hrefArr[1];
      }
    }
  }
}

// Check given item against blacklist, return null if in blacklist
const blacklist = ["comment"];
function checkAgainstBlacklist(elem, level) {
  if (elem && elem != null) {
    const className = elem.className,
      id = elem.id;

    const isBlackListed = blacklist
      .map((item) => {
        if (
          (typeof className === "string" && className.indexOf(item) >= 0) ||
          (typeof id === "string" && id.indexOf(item) >= 0)
        ) {
          return true;
        }
      })
      .filter((item) => item)[0];

    if (isBlackListed) {
      return null;
    }

    const parent = elem.parentElement;
    if (level > 0 && parent && !parent.isSameNode(document.body)) {
      return checkAgainstBlacklist(parent, --level);
    }
  }

  return elem;
}

/////////////////////////////////////
// Extension-related adder functions
/////////////////////////////////////

function checkPremium() {
  afterPremium();
}

function afterPremium() {
  continueLoading();
}

// Add our styles to the page
function addStylesheet(doc, link, classN) {
  const path = chrome.runtime.getURL(link),
    styleLink = document.createElement("link");

  styleLink.setAttribute("rel", "stylesheet");
  styleLink.setAttribute("type", "text/css");
  styleLink.setAttribute("href", path);

  if (classN) styleLink.className = classN;

  doc.head.appendChild(styleLink);
}

// Add the article author and date
function addArticleMeta() {

  const metaContainer = document.createElement("div");
  metaContainer.className = "simple-meta";
  const title = document.createElement("h1");

  const titleContent = document.createElement("div");

  title.className = "simple-title";

  titleContent.innerText = getArticleTitle();
  title.appendChild(titleContent);
  metaContainer.appendChild(title);

  return metaContainer;
}

// Add the close button
function addCloseButton() {
  let closeButton = document.createElement("button");
  closeButton.className = "simple-control simple-close";
  closeButton.title = "Close Just Read";
  closeButton.textContent = "x";

  return closeButton;
}


////////////////////////////
// Feature-related functions
////////////////////////////

// Add the theme editor button
//let datGUI;

function checkBreakpoints() {
  if (simpleArticleIframe) {
    let container = simpleArticleIframe.querySelector(
      ".simple-article-container"
    );
    if (window.innerWidth - container.offsetWidth < 320) {
      // Too small to show regular comments
      simpleArticleIframe.body.classList.add("simple-compact-view");
    } else {
      simpleArticleIframe.body.classList.remove("simple-compact-view");
    }
  }
}


let editBar;


/////////////////////////////////////
// Actually create the iframe
/////////////////////////////////////

let titleSelector,
  contentSelector,
  headerImageSelector,
  selectorsToDelete;

function createSimplifiedOverlay() {
  // Disable scroll on main page until closed
  document.documentElement.classList.add("simple-no-scroll");

  // Create an iframe so we don't use old styles
  simpleArticle = document.createElement("iframe");
  simpleArticle.id = "simple-article";
  simpleArticle.className = "simple-fade-up no-trans"; // Add fade

  const container = document.createElement("div");
  container.className = "simple-container";

  const articleContainer = document.createElement("div");
  articleContainer.className = "simple-article-container";

  // If there is no text selected, auto-select the content
  if (!pageSelectedContainer) {
    try {
      pageSelectedContainer = document.createElement("div");
      const doc = document.cloneNode(true);
      const readabilityParse = new Readability(doc, {
        charThreshold: 0,
      }).parse();

      const pattern = new RegExp("<br/?>[ \r\ns]*<br/?>", "g");
      pageSelectedContainer.innerHTML = DOMPurify.sanitize(
        readabilityParse.content.replace(pattern, "</p><p>")
      );
    } catch (e) {
      // If Readability.js fails, fallback to old method
      pageSelectedContainer = getArticleContainer();

      const pattern = new RegExp("<br/?>[ \r\ns]*<br/?>", "g");
      pageSelectedContainer.innerHTML = DOMPurify.sanitize(
        pageSelectedContainer.innerHTML.replace(pattern, "</p><p>")
      );
    }
  }

  selected = pageSelectedContainer;

  // Get the title, author, etc.
  articleContainer.appendChild(addArticleMeta());

  // Set the text as our text
  const contentContainer = document.createElement("div");
  contentContainer.className = "content-container";
  contentContainer.innerHTML = DOMPurify.sanitize(
    pageSelectedContainer.innerHTML
  );

  const lightboxes = [];

  const title = articleContainer.querySelector(".simple-title")?.textContent;

  // Strip inline styles
  const allElems = contentContainer.querySelectorAll("*");
  allElems.forEach((elem) => {
    if (elem != undefined) {
      elem.removeAttribute("style");
      elem.removeAttribute("color");
      elem.removeAttribute("width");
      elem.removeAttribute("height");
      elem.removeAttribute("background");
      elem.removeAttribute("bgcolor");
      elem.removeAttribute("border");

      // Delete the title if we find it in the article
      if (elem.textContent.trim() === title) {
        elem.parentElement.removeChild(elem);
      }

      // See if the pres have code in them
      let isPreNoCode = true;
      if (elem.nodeName === "PRE" && !leavePres) {
        isPreNoCode = false;

        Array.from(elem.children).forEach((child) => {
          if (child.nodeName === "CODE") {
            isPreNoCode = true;
          }
        });

        // If there's no code, format it
        if (!isPreNoCode) {
          elem.innerHTML = DOMPurify.sanitize(
            elem.innerHTML.replace(/\n/g, "<br/>")
          );
        }
      }

      // Replace the depreciated font element and pres without code with ps
      if ((elem.nodeName === "FONT" || !isPreNoCode) && elem.parentElement) {
        const p = document.createElement("p");
        p.innerHTML = DOMPurify.sanitize(elem.innerHTML);

        elem.parentElement.insertBefore(p, elem);
        elem.parentElement.removeChild(elem);
      }

      // Remove any inline style, LaTeX text, or noindex elements and things with aria hidden
      if (
        elem.nodeName === "STYLE" ||
        elem.nodeName === "NOINDEX" ||
        elem.nodeName === "LINK" ||
        elem.getAttribute("encoding") == "application/x-tex" ||
        (elem.getAttribute("aria-hidden") == "true" &&
          !elem.classList.contains("mwe-math-fallback-image-inline"))
      )
        elem.setAttribute("data-simple-delete", true);

      // Show LaTeX plain text on hover
      if (elem.classList.contains("mwe-math-fallback-image-inline")) {
        const plainText = document.createElement("div");
        plainText.className = "simple-plain-text";
        plainText.innerText = elem.alt;
        elem.parentElement.insertBefore(plainText, elem.nextSibling);
      }

      if (elem.nodeName === "IMG") {
        // Lightbox our images
        let img = elem;
        lightboxes.push(img);

        // Load lazy loaded images
        if (img.dataset.srcset) {
          img.srcset = img.dataset.srcset;
        } else if (img.dataset.src) {
          img.src = img.dataset.src;
        }
      }

    }
  });


  // Add the next chapter button if there is one
  const potentialOldMatches = [...contentContainer.querySelectorAll("a[href]")];
  if (
    !potentialOldMatches.some((match) => {
      const text = match.innerText.replace(/\s/g, "").toUpperCase();
      if (text === "NEXTCHAPTER" || text === "NEXT") {
        match.className = "jrNextChapter";
        return true;
      }
    })
  ) {
    const potentialNewMatches = [...document.querySelectorAll("a[href]")];

    potentialNewMatches.some((match) => {
      const text = match.innerText?.replace(/\s/g, "").toUpperCase();
      if (text === "NEXTCHAPTER" || text === "NEXT") {
        match.className = "jrNextChapter";
        contentContainer.appendChild(match);
        return true;
      }
    });
  }

  // Handle RTL sites
  const direction = window
    .getComputedStyle(document.body)
    .getPropertyValue("direction");
  if (
    direction === "rtl" ||
    (contentContainer.firstChild &&
      isRTL(contentContainer.firstChild.innerText))
  ) {
    container.classList.add("rtl");
  }

  articleContainer.appendChild(contentContainer);


  if (headerImageSelector && document.querySelector(headerImageSelector)) {
    const headerImg = document.querySelector(headerImageSelector);
    contentContainer.appendChild(headerImg);
  }

  // Create a container for the UI buttons
  let uiContainer = document.createElement("div");
  uiContainer.className = "simple-ui-container";

  // Add the close button
  uiContainer.appendChild(addCloseButton());

  container.appendChild(uiContainer);

  // Add our iframe to the page
  document.body.appendChild(simpleArticle);

  // Focus the article so our shortcuts work from the start
  document.getElementById("simple-article").focus();

  // Append our custom HTML to the iframe
  container.appendChild(articleContainer);

  function doStuff() {
    simpleArticleIframe =
      document.getElementById("simple-article").contentWindow.document;
    simpleArticleIframe.body.appendChild(container);
    simpleArticleIframe.documentElement.setAttribute(
      "lang",
      document.documentElement.getAttribute("lang")
    );

    simpleArticleIframe.body.className = window.location.hostname.replace(
      /\./g,
      "-"
    );


    // Flag any elements in the selectorsToDelete list for removal
    if (selectorsToDelete) {
      selectorsToDelete.forEach((selector) => {
        simpleArticleIframe.querySelectorAll(selector).forEach((elem) => {
          elem.dataset.simpleDelete = true; // Flag it for removal later
        });
      });
    }

    // Remove the elements we flagged earlier
    simpleArticleIframe
      .querySelectorAll("[data-simple-delete]")
      .forEach((elem) => {
        elem.parentElement.removeChild(elem);
      });


    // Add our listeners we need
    // The "X" button listener; exit if clicked
    simpleArticleIframe
      .querySelector(".simple-close")
      .addEventListener("click", closeOverlay);



    // Add lightboxes
    lightboxes.forEach((elem) => {
      // Wrap our image in a link
      const imgId = uuidv4();
      const wrapper = document.createElement("a");
      wrapper.href = "#" + imgId;
      if (elem.parentElement) {
        elem.parentElement.insertBefore(wrapper, elem);
        wrapper.appendChild(elem);

        // Create the lightbox version of our image
        const lightbox = document.createElement("a");
        lightbox.href = "#_";
        lightbox.className = "jr-lightbox";
        lightbox.id = imgId;

        const lightboxImg = document.createElement("img");
        lightboxImg.src = elem.src;
        lightbox.appendChild(lightboxImg);
        simpleArticleIframe
          .querySelector(".simple-container")
          .appendChild(lightbox);
      }
    });


    // Size our YouTube containers appropriately
    const youtubeFrames = simpleArticleIframe.querySelectorAll(
      "iframe[src *= 'youtube.com/embed/']"
    );
    youtubeFrames.forEach((frame) =>
      frame.parentElement.classList.add("youtubeContainer")
    );


    setTimeout(() => checkBreakpoints(), 10);

    finishLoading();
  }

  // Fix a bug in FF
  if (navigator.userAgent.toLowerCase().indexOf("firefox") > -1) {
    setTimeout(doStuff, 100);
  } else {
    doStuff();
  }
}

// Loads the styles after the xhr request finishes
let theme, styleElem;
function continueLoading() {
  // Create a style tag and place our styles in there from localStorage
  styleElem = document.createElement("style");

  // Create our version of the article
    createSimplifiedOverlay();
}

function fadeIn() {
  if (simpleArticleIframe.styleSheets.length > 2) {
    simpleArticle.classList.remove("no-trans");
    simpleArticle.classList.remove("simple-fade-up");

    // Remove contents of original page to make page more performant
    if (removeOrigContent) {
      simpleArticle.addEventListener(
        "transitionend",
        (e) => {
          [...document.body.children].forEach((child) =>
            child !== simpleArticle ? document.body.removeChild(child) : null
          );
        },
        { once: true }
      );
    }
  } else {
    setTimeout(fadeIn, 10);
  }
}

function onSimpleArticleIframeLoaded(cb) {
  if (simpleArticleIframe.readyState === "complete") {
    cb();
    return;
  }

  simpleArticleIframe.defaultView.addEventListener("load", cb);
}

function finishLoading() {
  // Add functionality for back button to close JR
  if (document.referrer !== window.location.href) {
    const url = new URL(window.location);
    window.history.pushState({}, "", url);
  }
  window.addEventListener("popstate", closeOverlay);

  // Add our required stylesheet for the articlž
  if (!simpleArticleIframe.head.querySelector(".required-styles"))
    addStylesheet(
      simpleArticleIframe,
      "required-styles.css",
      "required-styles"
    );

  addStylesheet(
      simpleArticleIframe,
      "dark-styles.css",
      "default-styles"
  );

  // Change the top most page when regular links are clicked
  for (let i = 0, l = simpleArticleIframe.links.length; i < l; i++) {
    simpleArticleIframe.links[i].onclick = linkListener;
  }

  // Navigate to the element specified by the URL # if it exists
  if (top.window.location.hash != null) {
    setTimeout(function () {
      simpleArticleIframe.location.hash = top.window.location.hash;
    }, 10);
  }

  // Append our theme styles to the overlay
  simpleArticleIframe.head.appendChild(styleElem);

  onSimpleArticleIframeLoaded(() => {
    chrome.runtime.sendMessage({ tabOpenedJR: window.location });
    fadeIn();
  });



  // Attempt to mute the elements on the original page
  mutePage();
  
}

function launch() {
  // Detect past overlay - don't show another
  if (document.getElementById("simple-article") == null) {
        addStylesheet(document, "page.css", "page-styles");
        checkPremium();
  } else {
    if (document.querySelector(".simple-fade-up") == null)
      // Make sure it's been able to load
      closeOverlay();
  }
}
    launch();
})();
